/*

  draw.c

  Drawing routines for the polyroids program.

    Copyright (C) 1993 Rob Shillingsburg
    Copyright (C) 2002 Luka Frelih <luka@mail.ljudmila.org>
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software

*/

#include "polyroids.h"

/*** Constants: **************************************************************/

#define SCORE_X 20         /* Where to print score and concussions. */
#define SCORE_Y 20
#define SHIELD_TEXT_X 475  /* Where to print the word 'shields:'.*/
#define SHIELD_TEXT_Y 20

#define SHIELD_LEFT 575    /* Where to put the shield gauge. */
#define SHIELD_BOTTOM 20
#define SHIELD_TOP 25
#define SHIELD_RIGHT (XSIZE - 20)

#define POWERUP_RADIUS 12

#define LINE_SPACING 40    /* Vertical distance between lines of text. */

/*** File-scope variables: ***************************************************/

char messages[2][255];
float seconds[2];

short triangle[3][2];  /* Vertices of a triangle. */
short square[4][2];    /* Vertices of a square. */
short pentagon[5][2];  /* Vertices of a pentagon. */


fmfonthandle onePointFont, thirtyPointFont;

/*** Internal function declarations: *****************************************/

void DrawPolyroids(void);
void DrawShip(void);
void DrawLaser(void);
void DrawAttractGraphics(void);
void DrawInfo(float deltaT);



       void
         charstr(char *string)
         {
           int len, i;
           len = (int) strlen(string);
           for (i = 0; i < len; i++) {
             glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18, string[i]);
           }
         }


void circ(float x, float y, GLdouble radius) {
    struct GLUquadric* q;
    pushmatrix();
    translate(x,y,0);
    q=gluNewQuadric();
    gluDisk(q, 0, radius, 20, 1);
    popmatrix();
}

void rot(float angle, char axis) {
    
    switch(axis) {
    
        case 'x':
        case 'X':
            glRotatef(angle, 1, 0, 0);
        break;
    
        case 'y':
        case 'Y':
            glRotatef(angle, 0, 1, 0);
        break;
    
        case 'z':
        case 'Z':
            glRotatef(angle, 0, 0, 1);
        break;
    }

}

/*** Interface function definitions: *****************************************/
 
/*
void GetFont(char *fontName)
{
  onePointFont = fmfindfont(fontName);
}
*/

/* Initializes drawing data. */
void DrawInit(void)
{
  int i;

  for (i = 0; i < 3; i++) {
    triangle[i][X] = POWERUP_RADIUS * cos(2 * PI / 3 * i);
    triangle[i][Y] = POWERUP_RADIUS * sin(2 * PI / 3 * i);
  }
  for (i = 0; i < 4; i++) {
    square[i][X] = POWERUP_RADIUS * cos(2 * PI / 4 * i);
    square[i][Y] = POWERUP_RADIUS * sin(2 * PI / 4 * i);
  }
  for (i = 0; i < 5; i++) {
    pentagon[i][X] = POWERUP_RADIUS * cos(2 * PI / 5 * i);
    pentagon[i][Y] = POWERUP_RADIUS * sin(2 * PI / 5 * i);
  }

/*
  fminit();
  if (!(onePointFont = fmfindfont("Honesty"))) {
    if (!(onePointFont = fmfindfont("Rock"))) {
      if (!(onePointFont = fmfindfont("Courier"))) {
	fprintf(stderr, "No font, can you install Courier?\n");
	exit(-1);
      }
    }
  }
*/
  thirtyPointFont = GLUT_BITMAP_HELVETICA_18;
}


void Draw(float deltaT)
{
  ClearWindow();
  DrawPolyroids();     /* Polyroids first, so they're in back. */
  if (!attractMode) {
    DrawShip();        /* Don't draw ship in attract mode. */
    DrawLaser();
  } else {
    DrawAttractGraphics();  /* Draw the attract graphics instead. */
  }
  DrawInfo(deltaT);          /* Info last, so it's in front. */
  swapbuffers();
}

/*** Internal function definitions: ******************************************/


/* We'll use these colors for the polyroids & lasers. 
static long polyroidColors[NUM_KINDS]
  = {RGB_RED, RGB_GREEN, RGB_YELLOW, RGB_CYAN};
*/

void setColorForKind(int kind) {

switch (kind) {

    case 0:
        cpack(RGB_RED);    
        break;
    case 1:
        cpack(RGB_GREEN);    
        break;
    case 2:
        cpack(RGB_YELLOW);    
        break;
    case 3:
        cpack(RGB_CYAN);    
        break;
    default:
        printf("WEIRD kind %d\n",kind);
}

}

void DrawPolyroids(void)
{
  int i, side;
  static int oddFrame = TRUE;

  oddFrame = !oddFrame;
  
  for (i = 0; i < numPolyroids; i++) {
    Polyroid *polyroid = polyroids[i];

    if (polyroid->dead) {
      cpack(RGB_MAGENTA);
    } else if (polyroid->beingHit && oddFrame) {
      cpack(RGB_WHITE);
    } else {
      setColorForKind(polyroid->kind);
    }

    bgnpolygon();
    for (side = 0; side < polyroid->numSides; side++) {
      v2s(polyroid->vertices[side]);
    }
    endpolygon();

    switch (polyroid->powerup) {
      case POW_NONE: case POW_NEVER: break;
      case POW_LASER: {
	pushmatrix();
	cpack(RGB_BLUE);
	translate(polyroid->centerX, polyroid->centerY, 0.0);
	rot(polyroid->angle / 2 / PI * 360, 'z');
	bgnpolygon();
	for (side = 0; side < 3; side++) {
	  v2s(triangle[side]);
	}
	endpolygon();
	popmatrix();
      } break;
      case POW_CONCUSSION: {
	pushmatrix();
	cpack(RGB_MAGENTA);
	translate(polyroid->centerX, polyroid->centerY, 0.0);
	rot(polyroid->angle / 2 / PI * 360, 'z');
	bgnpolygon();
	for (side = 0; side < 4; side++) {
	  v2s(square[side]);
	}
	endpolygon();
	popmatrix();
      } break;
      case POW_REGEN: {
	pushmatrix();
	cpack(RGB_BLACK);
	translate(polyroid->centerX, polyroid->centerY, 0.0);
	rot(polyroid->angle / 2 / PI * 360, 'z');
	bgnpolygon();
	for (side = 0; side < 5; side++) {
	  v2s(pentagon[side]);
	}
	endpolygon();
	popmatrix();
      } break;
    }    
  }
}



void DrawShip(void)
{
  int side;
  static int thrustColor = 0;

  /* Draw concussion wave. */
  if (concussionOn) {
    cpack(SHIP_COLOR);
    linewidth(2);
    circ(ship.centerX, ship.centerY,
	 (CONCUSSION_STEPS - concussionOn) * CONCUSSION_RADIUS);
    linewidth(1);
    concussionOn--;
  }
   
  /* Draw 'multicolor' thrust. */
  if (thrustOn) {
    setColorForKind(thrustColor);
    thrustColor = (thrustColor + 1) % NUM_KINDS;
    bgnpolygon();
    for (side = 0; side < 3; side++) {
      v2s(ship.thrustVertices[side]);
    }
    endpolygon();
  }

  /* Draw ship. */
  cpack(SHIP_COLOR);
  bgnpolygon();
  for (side = 0; side < 3; side++) {
    v2s(ship.vertices[side]);
  }
  endpolygon();
}



void DrawLaser(void)
{
  if (currentLaser != -1) {

    setColorForKind(currentLaser);

    linewidth(2.0);  /* 1-pixel line is almost invisible. */

    bgnline();
    v2s(ship.vertices[0]);
    v2s(laserEnd);
    endline();

    linewidth(1.0);
  }
}


/* Nice utility for centered text.  xCoord of -1 means center. 
   yCoord of -1 means skip to the next line. */
void DrawPositionedString(char *string, int xCoord, int yCoord)
{
  static int lastYCoord = YSIZE;

  if (yCoord != -1) lastYCoord = yCoord;

  if (xCoord == -1) {
    cmov2s(XSIZE / 2 - 0.5*fmgetstrwidth(thirtyPointFont, string), lastYCoord);
  } else {
    cmov2s(xCoord - 0.5*fmgetstrwidth(thirtyPointFont, string), lastYCoord);
  }

/*  fmsetfont(thirtyPointFont); */
  fmprstr(string);
}

/* Nice utility for centered text.  xCoord of -1 means center. 
   yCoord of -1 means skip to the next line. */
void DrawCenteredString(char *string, int yCoord)
{
  static int lastYCoord = YSIZE;

  if (yCoord == -1) lastYCoord -= LINE_SPACING;
  else lastYCoord = yCoord;

  cmov2s(XSIZE / 2 - 0.5*fmgetstrwidth(thirtyPointFont, string), lastYCoord);
/*  fmsetfont(thirtyPointFont); */
  fmprstr(string);
}

void DrawAttractGraphics(void)
{
  cpack(RGB_WHITE);

  if (showHelp) {
    /* Show "help" graphic. */
    DrawCenteredString("POLYROIDS", YSIZE - 140);
    DrawCenteredString("Copyright 1994 Rob Shillingsburg", -1);
//    DrawCenteredString("Adapted for OpenGL/GLUT 2002 Luka Frelih", -1);
    DrawCenteredString("", -1);
    DrawCenteredString("Destroy the evil polyroids before", -1);
    DrawCenteredString("they take over the universe!!", -1);
    DrawCenteredString("", -1);
/*
    DrawPositionedString("A", XSIZE / 4, -1);
    DrawPositionedString("Turn left", XSIZE / 4 + 100, -1);
    DrawPositionedString("J", 3 * XSIZE / 4, -1);
    DrawPositionedString("Fire red", 3 * XSIZE / 4 + 100, -1);*/


    DrawCenteredString("   A    Turn left      J    Fire red   ", -1);
    DrawCenteredString("   S     Thrust        K    Fire green ", -1);
    DrawCenteredString("   D     Turn right     L    Fire yellow", -1);
    DrawCenteredString("<SPACE>   Concussion!     ;    Fire cyan  ", -1);
    DrawCenteredString("", -1);
    DrawCenteredString("<RETURN> to start, <ESC> to quit", -1);
  } else {
    /* Show "high scores" graphic. */
    HSNode *node;

    DrawCenteredString("POLYROIDS HIGH SCORES", YSIZE - 80);
    DrawCenteredString("", -1);
    for (node = firstScore; node; node = node->next) {
      char buf[1024];

      sprintf(buf, "%d  %s", node->score, node->name);
      DrawCenteredString(buf, -1);
    }
  }
}


void DrawInfo(float deltaT)
{
  char string[100];   /* For sprintfing into. */ 
  static short shieldBarStart[2][2] = /* left side of shieldBar. */
    { {SHIELD_LEFT, SHIELD_TOP}, {SHIELD_LEFT, SHIELD_BOTTOM} };
  static short shieldBarEnd[2][2] =   /* right side of shieldBar. */
    { {SHIELD_RIGHT, SHIELD_TOP}, {SHIELD_RIGHT, SHIELD_BOTTOM} };
  short shieldBarMiddle[2][2];   /* Break point in shieldBar. */
  int i;

  cpack(RGB_WHITE);
/*
  if (numWavePolyroids == 0 && numLivePolyroids == 0) {
    sprintf(string, "END OF LEVEL %d", wave);
    DrawCenteredString(string, YSIZE / 2 + 100);
  }
*/
  for (i = 0; i < 2; i++) {
    seconds[i] -= deltaT;
    if (seconds[i] <= 0) seconds[i] = 0.0;
    else DrawCenteredString(messages[i], YSIZE / 2 + 100 - (i*50));
  }

  sprintf(string, "Score: %d  Concussions: %d  Laser: %d  Regen: %d",
	  ship.score, ship.concussions,
	  (int) (ship.shotThrust / INC_SHOT_THRUST),
	  (int) (ship.regenRate / INC_REGEN_RATE));
  cmov2s(SCORE_X, SCORE_Y);
  charstr(string);

  cmov2s(SHIELD_TEXT_X, SHIELD_TEXT_Y);
  charstr("Shields:");

  /* Find x coord where shield level is. */
  shieldBarMiddle[0][X] = shieldBarMiddle[1][X] =
    SHIELD_LEFT + (SHIELD_RIGHT - SHIELD_LEFT) * ship.shields;
  shieldBarMiddle[0][Y] = SHIELD_BOTTOM;
  shieldBarMiddle[1][Y] = SHIELD_TOP;

  /* Draw shield part. */
  bgnpolygon();
  v2s(shieldBarStart[0]);
  v2s(shieldBarStart[1]);
  v2s(shieldBarMiddle[0]);
  v2s(shieldBarMiddle[1]);
  endpolygon();
  
  /* Draw gauge background part. */
  bgnclosedline();
  v2s(shieldBarMiddle[0]);
  v2s(shieldBarMiddle[1]);
  v2s(shieldBarEnd[0]);
  v2s(shieldBarEnd[1]);
  endclosedline();
}




void SetMessage(int which, float howLong, char *msg)
{
  strcpy(messages[which], msg);
  seconds[which] = howLong;
}
