/*

  draw.js

  Drawing routines for the polyroids program.

    Copyright (C) 1993 Rob Shillingsburg
    Copyright (C) 2002,2009 Luka Frelih <luka@mail.ljudmila.org>
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software

*/

/*** Constants: **************************************************************/

SCORE_X = 20         /* Where to print score and concussions. */
SCORE_Y =  20
SHIELD_TEXT_X =  475  /* Where to print the word 'shields:'.*/
SHIELD_TEXT_Y =  20

SHIELD_LEFT =  575    /* Where to put the shield gauge. */
SHIELD_BOTTOM =  20
SHIELD_TOP =  25
SHIELD_RIGHT =  (XSIZE - 20)

POWERUP_RADIUS =  12

LINE_SPACING =  40    /* Vertical distance between lines of text. */

/*** File-scope variables: ***************************************************/

var messages=[];
var seconds=[0,0,0];

var triangle=[[],[],[]];  /* Vertices of a triangle. */
var square=[[],[],[],[]];    /* Vertices of a square. */
var pentagon=[[],[],[],[],[]];  /* Vertices of a pentagon. */


var onePointFont, thirtyPointFont;

/*** Interface function definitions: *****************************************/
 
/*
void GetFont(char *fontName)
{
  onePointFont = fmfindfont(fontName);
}
*/

/* Initializes drawing data. */
var c;

function DrawInit()
{

  c = document.getElementById('polyroids').getContext('2d');
  c.scale(1,-1);
  c.translate(0,-YSIZE);  
//konqueror  c.fillText=function(){};    
  for (i = 0; i < 3; i++) {
    triangle[i][X] = POWERUP_RADIUS * Math.cos(2 * PI / 3 * i);
    triangle[i][Y] = POWERUP_RADIUS * Math.sin(2 * PI / 3 * i);
  }

  for (i = 0; i < 4; i++) {
    square[i][X] = POWERUP_RADIUS * Math.cos(2 * PI / 4 * i);
    square[i][Y] = POWERUP_RADIUS * Math.sin(2 * PI / 4 * i);
  }
  for (i = 0; i < 5; i++) {
    pentagon[i][X] = POWERUP_RADIUS * Math.cos(2 * PI / 5 * i);
    pentagon[i][Y] = POWERUP_RADIUS * Math.sin(2 * PI / 5 * i);
  }

/*
  fminit();
  if (!(onePointFont = fmfindfont("Honesty"))) {
    if (!(onePointFont = fmfindfont("Rock"))) {
      if (!(onePointFont = fmfindfont("Courier"))) {
	fprintf(stderr, "No font, can you install Courier?\n");
	exit(-1);
      }
    }
  }
*/
  thirtyPointFont = "Swiss Cheesed";

}



function Draw(deltaT)
{
  if (conf['nodraw']) return;
  //console.log("draw");

  c.clearRect(0,0,XSIZE,YSIZE); // clear canvas

  DrawPolyroids();     /* Polyroids first, so they're in back. */

  if (!attractMode) {
    DrawShip();        /* Don't draw ship in attract mode. */
    DrawLaser();
  } else {
    DrawAttractGraphics();  /* Draw the attract graphics instead. */
  }
  if (conf['drawtext']) DrawInfo(deltaT);          /* Info last, so it's in front. */
  //swapbuffers();
}

/*** Internal function definitions: ******************************************/


// We'll use these colors for the polyroids & lasers. 

var polyroidColors = [RGB_RED, RGB_GREEN, RGB_YELLOW, RGB_CYAN];

function setColorForKind(kind) {
  c.fillStyle=c.strokeStyle=polyroidColors[kind];
}

var oddFrame = true;

function DrawPolyroids()
{
  //console.log("drawpolyroids");
  oddFrame = !oddFrame;
  
  for (i = 0; i < numPolyroids; i++) {
    var polyroid = polyroids[i];
    //console.log("drawing ",polyroid);
    if (!polyroid) {
     console.warn("oops in draw",i,numPolyroids,polyroids);
     continue; }

    if (polyroid.dead) {
      c.fillStyle= RGB_MAGENTA;
    } else if (polyroid.beingHit && oddFrame) {
      c.fillStyle= RGB_WHITE;
    } else {
      setColorForKind(polyroid.kind);
    }

    c.beginPath();
    c.moveTo(polyroid.vertices[0][0],polyroid.vertices[0][1]);
    for (side = 1; side < polyroid.numSides; side++) {
      c.lineTo(polyroid.vertices[side][0],polyroid.vertices[side][1]);
    }
    c.fill();

    switch (polyroid.powerup) {
      case POW_NONE: case POW_NEVER: break;
      case POW_LASER: {
    c.save();
	c.fillStyle= RGB_BLUE;
	c.translate(polyroid.centerX, polyroid.centerY);
	c.rotate(polyroid.angle);
    c.beginPath();
    c.moveTo(triangle[0][0],triangle[0][1]);
    for (side = 1; side < 3; side++) {
      c.lineTo(triangle[side][0],triangle[side][1]);
    }
    c.fill();
	c.restore();
      } break;
      case POW_CONCUSSION: {
    c.save();
	c.fillStyle= RGB_MAGENTA;
	c.translate(polyroid.centerX, polyroid.centerY);
	c.rotate(polyroid.angle);
    c.beginPath();
    c.moveTo(square[0][0],square[0][1]);
    for (side = 1; side < 4; side++) {
      c.lineTo(square[side][0],square[side][1]);
    }
    c.fill();
	c.restore();
      } break;
      case POW_REGEN: {
    c.save();
	c.fillStyle= RGB_BLACK;
	c.translate(polyroid.centerX, polyroid.centerY);
	c.rotate(polyroid.angle);
    c.beginPath();
    c.moveTo(pentagon[0][0],pentagon[0][1]);
    for (side = 1; side < 5; side++) {
      c.lineTo(pentagon[side][0],pentagon[side][1]);
    }
    c.fill();
	c.restore();
      } break;
    }    
  }
}


var thrustColor = 0;

function circ(x,y,radius) {
    c.arc(x,y,radius,0,Math.PI*2,true); // Outer circle
}

function DrawShip()
{
  //console.log("drawship",ship);
  /* Draw concussion wave. */
  if (concussionOn) {
    c.fillStyle= SHIP_COLOR;
    c.lineWidth=2;
    circ(ship.centerX, ship.centerY,
	 (CONCUSSION_STEPS - concussionOn) * CONCUSSION_RADIUS);
    c.lineWidth=1;
    concussionOn--;
  }
   
  /* Draw 'multicolor' thrust. */
  if (thrustOn) {
    setColorForKind(thrustColor);
    thrustColor = (thrustColor + 1) % NUM_KINDS;
    c.beginPath();
    c.moveTo(ship.thrustVertices[0][0],ship.thrustVertices[0][1]);
    c.lineTo(ship.thrustVertices[1][0],ship.thrustVertices[1][1]);
    c.lineTo(ship.thrustVertices[2][0],ship.thrustVertices[2][1]);
    c.fill();
  }

  //console.log(ship.vertices);
  /* Draw ship. */
  c.fillStyle= SHIP_COLOR;
  c.beginPath();
  c.moveTo(ship.vertices[0][0],ship.vertices[0][1]);
  c.lineTo(ship.vertices[1][0],ship.vertices[1][1]);
  c.lineTo(ship.vertices[2][0],ship.vertices[2][1]);
  c.fill();
}



function DrawLaser()
{
  if (currentLaser != -1) {

    setColorForKind(currentLaser);

    c.lineWidth=(2.0);  /* 1-pixel line is almost invisible. */

    c.beginPath();
    c.moveTo(ship.vertices[0][0],ship.vertices[0][1]);
    c.lineTo(laserEnd[0],laserEnd[1]);
    c.stroke();

    c.lineWidth=(1.0);
  }
}


/* Nice utility for centered text.  xCoord of -1 means center. 
   yCoord of -1 means skip to the next line. */
var lastYCoord = YSIZE;
function DrawPositionedString(string, xCoord, yCoord)
{
  c.textAlign="center";
  c.font="8pt sans-serif";
    
  if (yCoord != -1) lastYCoord = yCoord;

  c.save();
  c.translate(0,YSIZE);
  c.scale(1,-1);

  if (xCoord == -1) {
    c.fillText(string, XSIZE / 2, YSIZE-lastYCoord);    
  } else {
    c.fillText(string, xCoord, YSIZE-lastYCoord);
  }
  c.restore();

}

/* Nice utility for centered text.  xCoord of -1 means center. 
   yCoord of -1 means skip to the next line. */
function DrawCenteredString(string, yCoord)
{
  if (yCoord == -1) lastYCoord -= LINE_SPACING;
  else lastYCoord = yCoord;

  c.textAlign="center";  
  c.font=(conf['fancyfont']) ? "18pt Swiss Cheesed" : "18pt sans-serif";
  c.save();
  c.translate(0,YSIZE);
  c.scale(1,-1);
  c.fillText(string, XSIZE / 2, YSIZE-lastYCoord);
  c.restore();
}

function  DrawAttractGraphics()
{
  c.fillStyle= RGB_WHITE;

  if (showHelp) {
    /* Show "help" graphic. */
    DrawCenteredString("POLYROIDS beta", YSIZE - 140);
    DrawCenteredString("Created in 1994 by Rob Shillingsburg", -1);
    DrawCenteredString("Ported by Luka Frelih", -1);
    DrawCenteredString("", -1);
    DrawCenteredString("Destroy the evil polyroids before", -1);
    DrawCenteredString("they take over the universe!!", -1);
    DrawCenteredString("", -1);
/*
    DrawPositionedString("A", XSIZE / 4, -1);
    DrawPositionedString("Turn left", XSIZE / 4 + 100, -1);
    DrawPositionedString("J", 3 * XSIZE / 4, -1);
    DrawPositionedString("Fire red", 3 * XSIZE / 4 + 100, -1);*/


    DrawCenteredString("   A    Turn left      H    Fire red   ", -1);
    DrawCenteredString("   S     Thrust        J    Fire green ", -1);
    DrawCenteredString("   D     Turn right     K    Fire yellow", -1);
    DrawCenteredString("<SPACE>   Concussion!     L    Fire cyan  ", -1);
    DrawCenteredString("", -1);
    DrawCenteredString("<RETURN> to start, <ESC> to quit", -1);
  } else {
    /* Show "high scores" graphic. */

    DrawCenteredString("POLYROIDS HIGH SCORES", YSIZE - 80);
    DrawCenteredString("", -1);
/*    for (node = firstScore; node; node = node.next) {
      DrawCenteredString(""+node.score+" "+node.name, -1);
    }
*/  }
}


function  DrawInfo(deltaT)
{

  c.fillStyle= RGB_WHITE;
  c.strokeStyle= RGB_WHITE;

  for (i = 0; i < 2; i++) {
    seconds[i] -= deltaT;
    if (seconds[i] <= 0) seconds[i] = 0.0;
    else DrawCenteredString(messages[i], YSIZE / 2 + 100 - (i*50));
  }

  //console.log(seconds[0],seconds[1],seconds[2]);

  c.save();
  c.font=(conf['fancyfont']) ? "16pt Lavoisier" : "14pt sans-serif";

  c.textAlign="start";
  c.translate(0,YSIZE);
  c.scale(1,-1);
  c.fillText("Score: "+ship.score+"  Concussions: "+ship.concussions+"  Laser: "+Math.floor(ship.shotThrust / INC_SHOT_THRUST)+"  Regen: "+Math.floor(ship.regenRate / INC_REGEN_RATE), SCORE_X, YSIZE-SCORE_Y);
  

  c.fillText("Shields:",SHIELD_TEXT_X, YSIZE-SHIELD_TEXT_Y);
  c.restore();

  shieldBarMiddle=[[[],[]],[[],[]]]
  /* Find x coord where shield level is. */
  shieldBarMiddle[0][X] = shieldBarMiddle[1][X] =
    SHIELD_LEFT + (SHIELD_RIGHT - SHIELD_LEFT) * ship.shields;
  shieldBarMiddle[0][Y] = SHIELD_BOTTOM;
  shieldBarMiddle[1][Y] = SHIELD_TOP;

  /* Draw shield part. */
  c.fillRect(SHIELD_LEFT,SHIELD_BOTTOM,SHIELD_WIDTH * ship.shields,SHIELD_HEIGHT)  
  
  /* Draw gauge background part. */
  c.strokeRect(SHIELD_LEFT,SHIELD_BOTTOM,SHIELD_WIDTH,SHIELD_HEIGHT)  

}

SHIELD_WIDTH=SHIELD_RIGHT-SHIELD_LEFT;
SHIELD_HEIGHT=SHIELD_TOP-SHIELD_BOTTOM;

var msg;

function SetMessage(which, howLong, xmsg)
{   
  msg=messages[which]=xmsg
  //console.log("MESSAGE",which,",",howLong,",",xmsg)
  seconds[which] = howLong;
}
