/*

  header.js

  Header file for the polyroids program.

    Copyright (C) 1993 Rob Shillingsburg
    Copyright (C) 2002,2009 Luka Frelih <luka@mail.ljudmila.org>
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


*/

/*** Constants: **************************************************************/

PI = Math.PI
ROOT_2 = Math.sqrt(2)

X=0                 /* Indices for point arrays. */
Y=1

TRUE=1
FALSE=0

J_LASER=0            /* Indices for laser tables. */
K_LASER=1
L_LASER=2
SEMI_LASER=3

/* Window position and dimensions. */
XSIZE = 1024
YSIZE = 768
XPOS = 0
YPOS = 0

MAX_POLYROIDS = 20         /* Max number of polyroids on the screen. */
POLYROID_RADIUS = 35  /* Distance from center to vertex of a polyroid. */
POLYROID_MIN_RADIUS = 5    /* Min radius for a dead polyroid. */
SHRINK_SPEED = 8           /* In pixels per second. */
MAX_POLYROID_SIDES = 6     /* Max number of sides of a polyroid. */
MAX_POLYROID_SPIN = 4*PI   /* Spin rate when polyroid pops a new side. */
PER_SIDE_POLYROID_VELOCITY = 50  /* Velocity of a 1-sided polyroid. */
PER_SIDE_POLYROID_ACCELERATION = 100 /* Accel of a 1-sided polyroid. */

SHIP_RADIUS = 15            /* Size of the ship. */
SHIP_CORNER_ANGLE = 4*PI/5  /* Between 0 and PI */
SHIP_COLOR = "rgb(255,255,255)"      /* Ship's color. */
SHIP_THRUST = 500           /* Pixels per second per second. */
DRAG_COEFF = .4             /* Fraction of velocity lost per second. */
SHIP_TURN = 3*PI/2          /* Radians per second. */
MAX_SHIP_VELOCITY = 1000    /* Manhattan... */
BASE_SHOT_STRENGTH = 5*PI   /* Radians per second. */
INC_SHOT_STRENGTH = .25*PI   /* Radians per second. */
BASE_SHOT_THRUST = 500      /* Pixels per second per second. */
INC_SHOT_THRUST = 25        /* Pixels per second per second. */
LASER_MATCH_FACTOR = 2      /* Bonus to strength and thrust for
				     hitting with the proper laser. */
INITIAL_CONCUSSIONS = 3     /* Starting number of concussions. */
CONCUSSION_STEPS = 15       /* Number of waves in a concussion. */
CONCUSSION_RADIUS = 15      /* Distance between successive waves. */
EXPLOSION_STRENGTH = 100000 /* Pixels per second per second. */
BASE_REGEN_RATE = 0.006     /* Fraction of shield regained per second. */
INC_REGEN_RATE = 0.002      /* Fraction of shield regained per second. */
COLLISION_PENALTY = .75     /* Fraction of shield lost per second
				     of contact with a polyroid. */

NUM_KINDS = 4               /* Number of colors of polyroids & lasers. */
NUM_HIGH_SCORES = 14        /* Number of high scores to remember. */

POW_NONE = 0
POW_LASER = 1
POW_CONCUSSION = 2
POW_REGEN = 3
POW_NEVER = 4

/* Some rgb colors. */
RGB_BLACK = "rgb(0, 0, 0)"
RGB_RED	= "rgb(255, 0, 0)"
RGB_GREEN = "rgb(0, 255, 0)"
RGB_BLUE = "rgb(0, 0, 255)"
RGB_CYAN = "rgb(0, 255, 255)"
RGB_YELLOW = "rgb(255, 255, 0)"
RGB_MAGENTA = "rgb(255, 0, 255)"
RGB_WHITE = "rgb(255, 255, 255)"



/* A structure for holding a segment, including the slope. */
function Segment() {
  this.pt1=[];       /* Points to two shorts next to each other, x and y. */
  this.pt2=[];       /* Ditto. */
  this.slope=null;      /* Slope of the segment. */
}

