/*

  polyroids.js

  Main function for polyroids.

    Copyright (C) 1993 Rob Shillingsburg
    Copyright (C) 2002,2009 Luka Frelih <luka@mail.ljudmila.org>
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

//console={};
//console.log=function(){}
SCORES_DIR = "http://polyroids.ljudmila.org/scores/"
SCORES_FILE = "polyroids_scores"

/*** Global variable definitions: ********************************************/

/* The following are declared in polyroids.h.  Look there for description. */

wave = 1;
var polyroids=[0];
var numPolyroids = 0;
var numLivePolyroids = 0;
var numNewPolyroidsNeeded = 0;
var numWavePolyroids = 0;
var needNewPowerup = false;

var needRedraw = true;
var thrustOn = false;
var turningLeft = false;
var turningRight = false;
var laserOn = [false, false, false, false];
var concussionOn = false;
var currentLaser = -1;

var laserEnd=[];
var dead = false;
var attractMode = true;
var showHelp = true;

var firstScore = null; // hsNodes
var lastScore = null;

var conf = { size: 1024, fancyfont: true, 'interval': 33.33, 'drawtext': true  };

/*** File-scope variable definitions: ****************************************/

/* Records the time of the most recent frame. */
var lastFrameTime;

var ship = {};

//var laserOn = [];

/*** Interface function definitions: *****************************************/

function main()
{
  /* Setup the window. */
  Configure();

  AnimateIntervalID = setInterval(Animate, conf['interval']);  

}     

var inframe=false;
var elapsed=0;

function Animate()
{
    
    if (inframe) return;
    inframe=true;
    
    timeNow=(new Date).getTime();
    deltaT = (timeNow - lastFrameTime)/1000.0;
    lastFrameTime = timeNow;

    /* Move everything. */
    FindNewPolyroidPositions(deltaT);

    if (!attractMode) {
        FindNewShipPosition(deltaT);
        FindNewLaserPosition();
        CollisionDetect(deltaT);
    } else {
        elapsed += deltaT;
        if (elapsed > 10.0) {
            elapsed = 0.0;
       	    showHelp = !showHelp;
       	}
    }

  c.save();
  
  Draw(deltaT);

  c.restore();  
  
  inframe=false;
}





Reshape=function(width, height)
{
//  glViewport(0, 0, width, height);
//  glMatrixMode(GL_PROJECTION);
//  glLoadIdentity();
  ar = width / height;
//    glFrustum(-ar, ar, -1.0, 1.0, 2.0, 100.0);
//  glMatrixMode(GL_MODELVIEW);
//  glLoadIdentity();
}


function keydown(e)
{
    //console.log(e)
    //key = e.charCode?e.charCode:e.keyCode;
    HandleInput(e.keyCode,1);
    e.preventDefault();
//    e.stopPropagation();
}

document.onkeydown=keydown


function keyup(e)
{
    //console.log(e)
    //key = e.charCode?e.charCode:e.keyCode;
    HandleInput(e.keyCode,0);
    e.preventDefault();
//    e.stopPropagation();
}

document.onkeyup=keyup

/*** Internal function definitions: ******************************************/

ESCKEY=27
RETKEY=13
SPACEKEY=32
AKEY=65
SKEY=83
DKEY=68
JKEY=72
KKEY=74
LKEY=75
SEMICOLONKEY=76


function HandleInput(device, value)
{
  //console.log("INPUT on "+device+" "+value);

  if (attractMode) {
    switch(device) {
    case ESCKEY:
      if (value > 0) {
	exit(0);
      }
      break;      
    case RETKEY:
      attractMode = false;
      InitPolyroids(1);
      InitShip();
      SetMessage(0, 3.0, "LEVEL 1");
      break;
    case DKEY: 
        toggledebug(); 
    break;  
    case SPACEKEY: case AKEY: case SKEY: 
    case JKEY: case KKEY: case LKEY: case SEMICOLONKEY:
      if (value > 0) showHelp = !showHelp;
      break;
    default:
      break;
    }
  } else {  /* Not attract mode, ie, in the game. */
    switch(device) {
    case AKEY:    /* Keys A and D control turning left or right. */
      if (value > 0) {
	if (turningLeft) break;
	turningLeft = true;
	ship.dTheta += SHIP_TURN;
      } else {
	if (!turningLeft) break;
	turningLeft = false;
	ship.dTheta += -SHIP_TURN;
      }
      break;
    case SKEY:    /* S turns thrust on. */
      if (value > 0) {
	if (thrustOn) break;
	thrustOn = true;
	ship.ddx = SHIP_THRUST * Math.cos(ship.angle);
	ship.ddy = SHIP_THRUST * Math.sin(ship.angle);
      } else {
	if (!thrustOn) break;
	thrustOn = false;
	ship.ddx = 0; ship.ddy = 0;
      }
      break;
    case DKEY:      /* Keys A and D control turning left or right. */
      if (value > 0) {
	if (turningRight) break;
	turningRight = true;
	ship.dTheta -= SHIP_TURN;
      } else {
	if (!turningRight) break;
	turningRight = false;
	ship.dTheta -= -SHIP_TURN;
      }
      break;
      /* Keys J, K, L and ; turn on lasers.  We keep the lasers
	 in a stack, so that when the user releases a key, we
	 can change the laser to the most recently-pressed key
	 which is still held down. */
    case JKEY:
      if (value > 0) {
	if (laserOn[J_LASER]) break;
	PushLaser(J_LASER);
      } else {
	PopLaser(J_LASER);
      }
      break;
    case KKEY:
      if (value > 0) {
	if (laserOn[K_LASER]) break;
	PushLaser(K_LASER);
      } else {
	PopLaser(K_LASER);
      }
      break;
    case LKEY:
      if (value > 0) {
	if (laserOn[L_LASER]) break;
	PushLaser(L_LASER);
      } else {
	PopLaser(L_LASER);
      }
      break;
    case SEMICOLONKEY:
      if (value > 0) {
	if (laserOn[SEMI_LASER]) break;
	PushLaser(SEMI_LASER);
      } else {
	PopLaser(SEMI_LASER);
      }
      break;
    case SPACEKEY:   /* Space triggers a concussion shock. */
      if (value > 0) {
	if (!concussionOn && ship.concussions > 0) {
	  
	  for (i = 0; i < numPolyroids; i++) {
	    polyroid = polyroids[i];
	    xdist = polyroid.centerX - ship.centerX;
	    ydist = polyroid.centerY - ship.centerY;
	    distSquared = .01*xdist*xdist + .01*ydist*ydist;
	    /* Strength varies inversely with the square of the distance. */
	    strength = EXPLOSION_STRENGTH / distSquared;
	    angle = Math.atan2(ydist, xdist);
	    if (!(angle>0 || angle<=0)) {
	        console.log("angle!");
	    } 
	     
	    strength = bound(strength, 0, 10*PER_SIDE_POLYROID_VELOCITY);
	    polyroid.dx += strength * Math.cos(angle);
	    polyroid.dy += strength * Math.sin(angle);
	    polyroid.dTheta = 0;      /* Also stop everyone from spinning. */
	    if (!polyroid.dead) {
	      polyroid.numSides -= 1;   /* Reduce # of sides. */
	      if (polyroid.numSides < 3) {
       		KillPolyroid(polyroid);
	      }
	    }
	  }
	  ship.concussions--;
	  concussionOn = CONCUSSION_STEPS;  /* We use concussionOn as both
					       a counter and a flag. */
	}
      }
      break;
    case ESCKEY:  /* Quit the current game, going back to attract mode. */
      EndGame();
      break;
    default:
      break;
    }
  }
}



function Configure()
{
  /* Setup initial viewing coordinates * /
  ortho2(-0.5, XSIZE+0.5, -0.5, YSIZE+0.5 );

  /* Initialize draw data & font manager. */
  DrawInit();

  /* Record starting time. */
  lastFrameTime= (new Date).getTime();  
  //srand(lastFrameTime.tv_sec);

  /* Initialize the game. */
  InitPolyroids((Math.random() * MAX_POLYROIDS) / 2 + 1);
  InitShip();
  //ReadHighScores();
  SetMessage(0, 0.0, "");
  SetMessage(1, 0.0, "");
}

function GetPos(polyroid)
{
  pos = Math.floor(Math.random() * 8);

  switch (pos) {
    case 0: {
      polyroid.centerX = 0;
      polyroid.centerY = 0;
    } break;
    case 1: {
      polyroid.centerX = 0;
      polyroid.centerY = YSIZE / 2;
    } break;
    case 2: {
      polyroid.centerX = 0;
      polyroid.centerY = YSIZE;
    } break;
    case 3: {
      polyroid.centerX = XSIZE / 2;
      polyroid.centerY = 0;
    } break;
    case 4: {
      polyroid.centerX = XSIZE / 2;
      polyroid.centerY = YSIZE;
    } break;
    case 5: {
      polyroid.centerX = XSIZE;
      polyroid.centerY = 0;
    } break;
    case 6: {
      polyroid.centerX = XSIZE;
      polyroid.centerY = YSIZE / 2;
    } break;
    case 7: {
      polyroid.centerX = XSIZE;
      polyroid.centerY = YSIZE;
    } break;
    default: {
      polyroid.centerX=0;
      polyroid.centerY=0;
    }
  }

    //console.log(polyroid.centerX,polyroid.centerY);
}


/* Makes a new, random polyroid. */
function MakeNewPolyroid()
{
    
  polyroid = {};

  polyroid.kind = Math.floor(Math.random() * 4);

  polyroid.radius = POLYROID_RADIUS;
  polyroid.angle = 0;
  GetPos(polyroid);
  polyroid.dx = Math.floor(Math.random() * 256);
  polyroid.dy = Math.floor(Math.random() * 256);
  polyroid.dTheta = 0;
  polyroid.ddx = 0;
  polyroid.ddy = 0;
  polyroid.ddTheta = (Math.floor(Math.random() * 4) + 1) * PI/10;
  polyroid.numSides = Math.floor(Math.random() * 4) + 3;
  polyroid.dead = false;
  polyroid.beingHit = false;
  //  polyroid.chaseFactor = (rand() & 1023) + 1; 
  polyroid.chaseFactor = Math.random();
  polyroid.powerup = POW_NEVER;
  polyroid.vertices=[[],[],[],[],[],[],[],[],[],[],[],[]];

  //console.log("new polyroid ",polyroid)
  return polyroid;
}


function NextPowerup(current)
{
  switch (current) {
    case POW_LASER: {
      return POW_CONCUSSION;
    } break;
    case POW_CONCUSSION: {
      return POW_REGEN;
    } break;
    case POW_REGEN: {
      return POW_LASER;
    } break;
    case POW_NEVER: {
      return POW_NEVER;
    } break;
    default: exit(-1); break;
  }
}



function InitPolyroids(number)
{
  //console.log("InitPolyroids "+number);

  /* Free any old polyroids. */
  polyroids=[];  
  numPolyroids = numLivePolyroids = 0;
  numNewPolyroidsNeeded = number;
  needNewPowerup = true;
  
}


function InitShip () {

  //console.log("InitShip");

  ship={};
    
  ship.centerX = XSIZE / 2;
  ship.centerY = YSIZE / 2;
  ship.angle = 0;
  ship.dx = ship.dy = ship.dTheta = 0;
  ship.ddx = ship.ddy = 0;
  ship.score = 0;
  ship.concussions = INITIAL_CONCUSSIONS;
  ship.shields = 1.0;
  ship.shotStrength = BASE_SHOT_STRENGTH;
  ship.shotThrust = BASE_SHOT_THRUST;
  ship.regenRate = BASE_REGEN_RATE;
  ship.vertices=[[],[],[]];
  ship.thrustVertices=[[],[],[]];
  thrustOn = turningLeft = turningRight = false;
  for (i = 0; i < NUM_KINDS; i ++) laserOn[i] = false;
  ClearLaserStack();
  currentLaser = -1;

  wave = 1;
  numWavePolyroids = 0;
}



function EndGame()
{
  //console.log("EndGame");

  attractMode = true;

  //RIPPED OUT highscores    
  
  for (i = 0; i < numPolyroids; i++) {
    polyroid=polyroids[i];
    if (polyroid) {
        polyroid.beingHit = false;    
    }
  }

  SetMessage(0, 0.0, "");
  SetMessage(1, 0.0, "");

  showHelp = false;
}




function KillPolyroid(polyroid)
{
  //console.log("KillPolyroid "+polyroid);

  numLivePolyroids--;
  polyroid.numSides = 3;
  polyroid.dead = true;
  ship.score++;

  switch (polyroid.powerup) {
    case POW_NONE: 
    case POW_NEVER: 
        break;
    case POW_LASER: 
        ship.shotStrength += INC_SHOT_STRENGTH;
        ship.shotThrust += INC_SHOT_THRUST;
        SetMessage(1, 2.0, "LASER POWERUP");
        needNewPowerup = true;
        break;
    case POW_CONCUSSION: 
        ship.concussions++;
        SetMessage(1, 2.0, "CONCUSSION");
        needNewPowerup = true;
        break;
        
    case POW_REGEN: 
        ship.regenRate += INC_REGEN_RATE;
        SetMessage(1, 2.0, "INCREASED SHIELD REGENERATION");
        needNewPowerup = true;
        break;
  }
  polyroid.powerup = POW_NEVER;  

  if (numLivePolyroids == 0 && numWavePolyroids == 0) {
    SetMessage(0, 100.0, "END OF LEVEL "+wave);
  }

}

