/*

  position.js

  Position-calculation routines for polyroids.


    Copyright (C) 1993 Rob Shillingsburg
    Copyright (C) 2009 Luka Frelih <luka@mail.ljudmila.org>
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


*/


/*** Interface function definitions: *****************************************/



function assert(where) {
  
  if (numPolyroids!=polyroids.length) {
   console.error(where,'length',numPolyroids,polyroids.length);
   missing=numPolyroids-polyroids.length; 
   numPolyroids=polyroids.length;
   numNewPolyroidsNeeded+=missing;
   numWavePolyroids+=missing;
  }

  if (numLivePolyroids>numPolyroids) {
    console.error(where,'oops',numLivePolyroids,numPolyroids);
    numLivePolyroids=numPolyroids;
  }
}

var prevPowerup = POW_LASER;

function FindNewPolyroidPositions(deltaT)
{
  
  if (numWavePolyroids == 0 &&
      numNewPolyroidsNeeded == 0 && numPolyroids == 0) {
    wave++;
    ship.shields = 1.0;
    numWavePolyroids = wave*wave - wave;
    InitPolyroids(wave);
    needNewPowerup = TRUE;
    SetMessage(0, 3.0, "LEVEL "+wave);
  }

  var i;  

  /* Make new polyroids, if we need them. */
  if (numNewPolyroidsNeeded) {
    for (i = 0; (numPolyroids < MAX_POLYROIDS) && (i < numNewPolyroidsNeeded); i++) {
      //console.log("needed",numNewPolyroidsNeeded);
      polyroids[polyroids.length]=MakeNewPolyroid();
      numPolyroids++;        
      //console.log("ROID",polyroids[numPolyroids - 1]);
      numLivePolyroids++;
      if (((Math.random() * 0x10000) < (0xffff / wave / 3))) {
    	prevPowerup = NextPowerup(prevPowerup);
	    polyroids[numPolyroids - 1].powerup = prevPowerup;
	    needNewPowerup = FALSE;
      }
    }
    numNewPolyroidsNeeded = 0;
  }


  for (i = 0; i < numPolyroids; i++) {
    var polyroid = polyroids[i];
    
    if (conf['debug']) {assert('D');}
    
    if (!polyroid) { 
       console.warn("missing", numPolyroids, polyroids.length);
        continue; 
    }

    /* Shrink polyroid if dead. */
    if (polyroid.dead) {
      polyroid.radius -= deltaT * SHRINK_SPEED;
      if (polyroid.radius < POLYROID_MIN_RADIUS) {
    	if (numWavePolyroids > 0) {
    	  numNewPolyroidsNeeded++;
    	  numWavePolyroids--;
    	}
       	numPolyroids--;
     	//console.log("DEAD",i,polyroids[i]);
     	polyroids.splice(i,1);
        i--;
    	continue;
      }
    }

    /* Find angle along which we'll accelerate. */
    if (attractMode) {  /* Use different accels for attract mode. */
      angle = Math.atan2(ship.centerY - polyroid.centerY, ship.centerX - polyroid.centerX);
    } else {
      /*angle = atan2((ship.centerY - polyroid.centerY)
		    * polyroid.chaseFactor - polyroid.dy,
		    (ship.centerX - polyroid.centerX)
		    * polyroid.chaseFactor - polyroid.dx);*/
      /* Try: d = distToShip, v = distance per second
	 chaseFactor = (d >= v) ? 1 : d/v; */
   
      angle = Math.atan2(
        (ship.centerY - (polyroid.centerY + polyroid.dy * polyroid.chaseFactor)), 
        (ship.centerX - (polyroid.centerX + polyroid.dx * polyroid.chaseFactor))
        );
    }


    if (!(angle>0 || angle<=0)) { 
        angle=Math.PI/2; 
        console.warn("ANGLE NaN");
        console.log(ship.centerY - (polyroid.centerY + polyroid.dy * polyroid.chaseFactor));
        console.log(ship.centerX - (polyroid.centerX + polyroid.dx * polyroid.chaseFactor));    
     }

    //console.log(ship.centerX,polyroid.centerX);
    //console.log(ship.centerY,polyroid.centerY);
    //console.log("angle",angle,typeof(angle),angle>0,angle<=0);

    /* Find acceleration. */
    polyroid.ddx = polyroid.numSides * PER_SIDE_POLYROID_ACCELERATION * Math.cos(angle);
    polyroid.ddy = polyroid.numSides * PER_SIDE_POLYROID_ACCELERATION * Math.sin(angle);

    /* Find component velocities. */
    newDx = polyroid.dx + polyroid.ddx * deltaT;
    newDy = polyroid.dy + polyroid.ddy * deltaT;
    
    /* Find vectored velocity and speed limit. */
    newVelocity = Math.sqrt(newDy*newDy + newDx*newDx);
    allowedVelocity = bound(newVelocity, polyroid.numSides * -PER_SIDE_POLYROID_VELOCITY, polyroid.numSides * PER_SIDE_POLYROID_VELOCITY);
    
    /* If not exceeding speed limit, commit to new velocity,
       with drag adjustment. */
    /* If exceeding speed limit, use old velocity. */
    if (allowedVelocity == newVelocity) {
      polyroid.dx = newDx;
      polyroid.dy = newDy;
    }
    
      assert('h');

    /* Find position, checking for edge conditions and enforcing speed
       limit on bounces.  Also advance the powerup. */
    polyroid.centerX += polyroid.dx * deltaT;
    if (bound(polyroid.centerX, 0, XSIZE) != polyroid.centerX) {
      polyroid.centerX = bound(polyroid.centerX, 0, XSIZE);
      polyroid.dx = -newDx * allowedVelocity / newVelocity;
      polyroid.dy = newDy * allowedVelocity / newVelocity;
      if (polyroid.powerup != POW_NEVER)
	polyroid.powerup = NextPowerup(polyroid.powerup);
    }
    polyroid.centerY += polyroid.dy * deltaT;
    if (bound(polyroid.centerY, 0, YSIZE) != polyroid.centerY) {
      polyroid.centerY = bound(polyroid.centerY, 0, YSIZE);
      polyroid.dx = newDx * allowedVelocity / newVelocity;
      polyroid.dy = -newDy * allowedVelocity / newVelocity;
      if (polyroid.powerup != POW_NEVER)
	polyroid.powerup = NextPowerup(polyroid.powerup);
    }

    /* Find new rotational velocity, checking against speed limit. */
    polyroid.dTheta += polyroid.ddTheta * deltaT;
    if (Math.abs(polyroid.dTheta) > MAX_POLYROID_SPIN && !polyroid.dead) {
      if (polyroid.numSides < MAX_POLYROID_SIDES) polyroid.numSides++;
      polyroid.dTheta = 0;
      polyroid.ddTheta = -polyroid.ddTheta;
    }
    

    /* Find new angle, mod 2*PI. */
    polyroid.angle += polyroid.dTheta * deltaT;
    if (polyroid.angle > 2*PI) polyroid.angle += -2*PI;
    if (polyroid.angle < -2*PI) polyroid.angle += 2*PI;

    /* Calculate vertices. */
    for (side = 0; side < polyroid.numSides; side++) {

      baseAngle =  2 * PI * side / polyroid.numSides;

      polyroid.vertices[side][X] = polyroid.centerX + polyroid.radius * Math.cos(polyroid.angle + baseAngle);
      polyroid.vertices[side][Y] = polyroid.centerY + polyroid.radius * Math.sin(polyroid.angle + baseAngle);
    }
    
  }

}


function FindNewShipPosition(deltaT)
{
  //console.log("before:",ship)

  /* Increase shields a little. */
  ship.shields = bound(ship.shields + deltaT * ship.regenRate, 0, 1);
  
  /* Find new angle, mod 2*PI. */
  ship.angle += ship.dTheta * deltaT;
  if (ship.angle >= 2*PI) ship.angle += -2*PI;
  if (ship.angle < 0) ship.angle += 2*PI;
  
  /* Find new acceleration, taking angle into account. */
  if (thrustOn) {
    ship.ddx = SHIP_THRUST * Math.cos(ship.angle);
    ship.ddy = SHIP_THRUST * Math.sin(ship.angle);
  }

  
  /* Find new velocities and positions, enforcing speed limit on bounces. */
  ship.dx += ship.ddx * deltaT - DRAG_COEFF * deltaT * ship.dx;
  ship.centerX += ship.dx * deltaT;
  if (ship.centerX != bound(ship.centerX, 0, XSIZE)) {
    ship.centerX = bound(ship.centerX, 0, XSIZE);
    ship.dx = bound(-ship.dx, -MAX_SHIP_VELOCITY, MAX_SHIP_VELOCITY);
  }
  ship.dy += ship.ddy * deltaT - DRAG_COEFF * deltaT * ship.dy;
  ship.centerY += ship.dy * deltaT;
  if (ship.centerY != bound(ship.centerY, 0, YSIZE)) {
    ship.centerY = bound(ship.centerY, 0, YSIZE);
    ship.dy = bound(-ship.dy, -MAX_SHIP_VELOCITY, MAX_SHIP_VELOCITY);
  }

  
  /* Nose. */
  ship.vertices[0][X] = ship.centerX + SHIP_RADIUS * Math.cos(ship.angle);
  ship.vertices[0][Y] = ship.centerY + SHIP_RADIUS * Math.sin(ship.angle);

  /* Left corner. */
  ship.vertices[1][X] = ship.centerX + SHIP_RADIUS * Math.cos(ship.angle + SHIP_CORNER_ANGLE);
  ship.vertices[1][Y] = ship.centerY + SHIP_RADIUS * Math.sin(ship.angle + SHIP_CORNER_ANGLE);

  /* Right corner. */
  ship.vertices[2][X] = ship.centerX + SHIP_RADIUS * Math.cos(ship.angle - SHIP_CORNER_ANGLE);
  ship.vertices[2][Y] = ship.centerY + SHIP_RADIUS * Math.sin(ship.angle - SHIP_CORNER_ANGLE);

  /* Thrust tail. */
  ship.thrustVertices[0][X] = ship.centerX + SHIP_RADIUS * 2 * Math.cos(ship.angle + PI);
  ship.thrustVertices[0][Y] = ship.centerY + SHIP_RADIUS * 2 * Math.sin(ship.angle + PI);

  /* Thrust left corner. */
  ship.thrustVertices[1][X] = ship.centerX + SHIP_RADIUS * Math.cos(ship.angle + SHIP_CORNER_ANGLE + (PI - SHIP_CORNER_ANGLE) / 2);
  ship.thrustVertices[1][Y] = ship.centerY + SHIP_RADIUS * Math.sin(ship.angle + SHIP_CORNER_ANGLE + (PI - SHIP_CORNER_ANGLE) / 2);

  /* Thrust right corner. */
  ship.thrustVertices[2][X] = ship.centerX + SHIP_RADIUS * Math.cos(ship.angle - SHIP_CORNER_ANGLE - (PI - SHIP_CORNER_ANGLE) / 2);
  ship.thrustVertices[2][Y] = ship.centerY + SHIP_RADIUS * Math.sin(ship.angle - SHIP_CORNER_ANGLE - (PI - SHIP_CORNER_ANGLE) / 2);
}


function FindNewLaserPosition()
{
  if (currentLaser != -1) {
    tanAngle = Math.tan(ship.angle);  /* Cache this value. */

    if (tanAngle == 0.0) {  /* Straight across. */
      laserEnd[X] = (ship.angle == 0.0) ? XSIZE : 0;
      laserEnd[Y] = ship.centerY;
    } else {  /* Find intercept of laser with whichever wall. */
      distToTop = (ship.angle < PI) ?
	YSIZE - ship.centerY : -ship.centerY;
      topIntersectX = ship.centerX + distToTop / tanAngle;
      
      if (topIntersectX <= 0) {
      	laserEnd[X] = 0;
    	laserEnd[Y] = ship.centerY - ship.centerX * tanAngle;
      } else if (topIntersectX > XSIZE) {
    	laserEnd[X] = XSIZE;
    	laserEnd[Y] = ship.centerY + (XSIZE - ship.centerX) * tanAngle;
      } else {
    	laserEnd[X] = topIntersectX;
    	laserEnd[Y] = (ship.angle < PI) ? YSIZE : 0;
      }        
    }
  }
}
