/*

  util.js

  Utility functions for the polyroids program.


    Copyright (C) 1993 Rob Shillingsburg
    Copyright (C) 2009 Luka Frelih <luka@mail.ljudmila.org>
        
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


*/

/* Returns x, bounded by the given values. */
function bound(x, bottomBound, topBound)
{
  if (x > topBound) return topBound;
  else if (x < bottomBound) return bottomBound;
  else return x;
}

/* TRUE if pt is `above' seg. */
function Above(seg, pt)
{
  return (pt[Y] - seg.pt2[Y]) > seg.slope * (pt[X] - seg.pt2[X]);
}

/* TRUE if the segments intersect. */
function SegmentsIntersect(seg1, seg2)
{
  return (Above(seg1, seg2.pt1) != Above(seg1, seg2.pt2)) &&
    (Above(seg2, seg1.pt1) != Above(seg2, seg1.pt2));  
}

/* Returns the slope of the line containing the points. */
/* Gives a large number for vertical lines. */
function CalculateSlope(pt1, pt2)
{
  if (pt1 && pt2) {  
  rise = pt1[Y] - pt2[Y];
  run = pt1[X] - pt2[X];

  if (run == 0) return 2000.0;
  else return rise / run;
  }
}


laserStack = [-1];
top = 0;

function ClearLaserStack()
{
  top = 0;
}

/* Puts whichLaser on top of the laser stack.  We assume whichLaser is
   not already in the stack. */
function PushLaser(whichLaser)
{
  laserOn[whichLaser] = TRUE;
  laserStack[++top] = whichLaser;
  currentLaser = whichLaser;
}

/* Removes whichLaser from the stack, wherever it may be. */
function PopLaser(whichLaser)
{
  laserOn[whichLaser] = FALSE;
  
  for (i = top; i > 0; i--) {
    if (laserStack[i] == whichLaser) {
      for (j = i; j < top; j++) laserStack[j] = laserStack[j+1];
      top--;
    }
  }
  currentLaser = laserStack[top];
}
