/*

  polyroids.c

  Main function for polyroids.

    Copyright (C) 1993 Rob Shillingsburg
    Copyright (C) 2002 Luka Frelih <luka@mail.ljudmila.org>
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "polyroids.h"
#include <string.h>
#include <sys/time.h>
#include <unistd.h>
#include <pwd.h>


#ifndef SCORES_DIR
#define SCORES_DIR "/var/tmp"
#endif

#define SCORES_FILE "polyroids_scores"



/*** Global variable definitions: ********************************************/

/* The following are declared in polyroids.h.  Look there for description. */

int wave = 1;
Polyroid *polyroids[MAX_POLYROIDS];
int numPolyroids = 0, numLivePolyroids = 0;
int numNewPolyroidsNeeded = 0;
int numWavePolyroids = 0;
Boolean needNewPowerup = FALSE;
Ship ship;

int needRedraw = TRUE;
int thrustOn = FALSE;
int turningLeft = FALSE;
int turningRight = FALSE;
int laserOn[4] = {FALSE, FALSE, FALSE, FALSE};
int concussionOn = FALSE;
int currentLaser = -1;

short laserEnd[2];
int dead = FALSE;
int attractMode = TRUE;
int showHelp = TRUE;

HSNode *firstScore = NULL;
HSNode *lastScore = NULL;

/*** File-scope variable definitions: ****************************************/

/* Handles on the program's window. */
long squarewin, Curwin;

/* Records the time of the most recent frame. */
struct timeval lastFrameTime;

/*** Internal function declarations: *****************************************/

void InitPolyroids(int number);
void InitShip(void);
void ReadHighScores(void);
void WriteHighScores(void);
void HandleInput(long device, short value);

/*** Interface function definitions: *****************************************/

int main(int argc, char *argv[])
{
  glutInit(&argc, argv);

  /* Setup the window. */
  Configure();

  glutMainLoop();
  return 0;             /* ANSI C requires main to return int. */
}     




/*** GLUT callback definitions: ******************************************/

void
Animate(void)
{

    if (1||needRedraw) {
      struct timeval timeNow;
      float deltaT;               /* Frame gap in seconds. */

      while (gettimeofday(&timeNow, 0));  /* While loop for safety's sake. */
      deltaT = (timeNow.tv_sec - lastFrameTime.tv_sec) +
	(timeNow.tv_usec - lastFrameTime.tv_usec)/1000000.0;
      lastFrameTime = timeNow;

      /* Move everything. */
      FindNewPolyroidPositions(deltaT);
      if (!attractMode) {
	FindNewShipPosition(deltaT);
	FindNewLaserPosition();
	CollisionDetect(deltaT);
      } else {
	static float seconds = 0;

	seconds += deltaT;
	if (seconds > 10.0) {
	  seconds = 0.0;
	  showHelp = !showHelp;
	}
      }

      /* Draw everything. */
  //    Draw(deltaT);
      needRedraw = FALSE;
    }
  

   glutPostRedisplay();

}



static void 
Display(void)
{
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  /* viewing */
  glPushMatrix();

  Draw(0.01);

  glPopMatrix();

  glutSwapBuffers();
}

static void 
Reshape(int width, int height)
{
//fix with polyroids init
  float ar;
  glViewport(0, 0, width, height);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  ar = (float) width / (float) height;
  glFrustum(-ar, ar, -1.0, 1.0, 2.0, 100.0);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
}

static void 
Key(unsigned char key, int x, int y)
{
    HandleInput(key,1);
}

static void 
SpecialKey(int key, int x, int y)
{
    HandleInput(256+key,1);
}


static void 
KeyUp(unsigned char key, int x, int y)
{
    HandleInput(key,0);
}

static void 
SpecialKeyUp(int key, int x, int y)
{
    HandleInput(256+key,0);
}

void
Visible(int state)
{
    if (state == GLUT_VISIBLE) {
            glutIdleFunc(Animate);
    } else {
            glutIdleFunc(NULL);
    }
}

/*** Internal function definitions: ******************************************/


void HandleInput(long device, short value)
{
//  printf("INPUT on %ld: %d\n",device,value);

  if (attractMode) {
    switch(device) {
    case TIMER0:
    case REDRAW:
      needRedraw = TRUE;
      break;
    case INPUTCHANGE:
      Curwin = value;
      break;
    case ESCKEY:
      if (value > 0) {
	exit(0);
      }
      break;      
    case RETKEY:
      attractMode = FALSE;
      InitPolyroids(1);
      InitShip();
      SetMessage(0, 3.0, "LEVEL 1");
      break;
    case SPACEKEY: case AKEY: case SKEY: case DKEY:
    case JKEY: case KKEY: case LKEY: case SEMICOLONKEY:
      if (value > 0) showHelp = !showHelp;
      break;
    default:
      break;
    }
  } else {  /* Not attract mode, ie, in the game. */
    switch(device) {
    case TIMER0:
    case REDRAW:
      needRedraw = TRUE;
      break;
    case INPUTCHANGE:
      Curwin = value;
      break;
    case AKEY:    /* Keys A and D control turning left or right. */
      if (value > 0) {
	if (turningLeft) break;
	turningLeft = TRUE;
	ship.dTheta += SHIP_TURN;
      } else {
	if (!turningLeft) break;
	turningLeft = FALSE;
	ship.dTheta += -SHIP_TURN;
      }
      break;
    case SKEY:    /* S turns thrust on. */
      if (value > 0) {
	if (thrustOn) break;
	thrustOn = TRUE;
	ship.ddx = SHIP_THRUST * cos(ship.angle);
	ship.ddy = SHIP_THRUST * sin(ship.angle);
      } else {
	if (!thrustOn) break;
	thrustOn = FALSE;
	ship.ddx = 0; ship.ddy = 0;
      }
      break;
    case DKEY:      /* Keys A and D control turning left or right. */
      if (value > 0) {
	if (turningRight) break;
	turningRight = TRUE;
	ship.dTheta -= SHIP_TURN;
      } else {
	if (!turningRight) break;
	turningRight = FALSE;
	ship.dTheta -= -SHIP_TURN;
      }
      break;
      /* Keys J, K, L and ; turn on lasers.  We keep the lasers
	 in a stack, so that when the user releases a key, we
	 can change the laser to the most recently-pressed key
	 which is still held down. */
    case JKEY:
      if (value > 0) {
	if (laserOn[J_LASER]) break;
	PushLaser(J_LASER);
      } else {
	PopLaser(J_LASER);
      }
      break;
    case KKEY:
      if (value > 0) {
	if (laserOn[K_LASER]) break;
	PushLaser(K_LASER);
      } else {
	PopLaser(K_LASER);
      }
      break;
    case LKEY:
      if (value > 0) {
	if (laserOn[L_LASER]) break;
	PushLaser(L_LASER);
      } else {
	PopLaser(L_LASER);
      }
      break;
    case SEMICOLONKEY:
      if (value > 0) {
	if (laserOn[SEMI_LASER]) break;
	PushLaser(SEMI_LASER);
      } else {
	PopLaser(SEMI_LASER);
      }
      break;
    case SPACEKEY:   /* Space triggers a concussion shock. */
      if (value > 0) {
	if (!concussionOn && ship.concussions > 0) {
	  int i;
	  
	  for (i = 0; i < numPolyroids; i++) {
	    Polyroid *polyroid = polyroids[i];
	    int xdist = polyroid->centerX - ship.centerX;
	    int ydist = polyroid->centerY - ship.centerY;
	    float distSquared = .01*xdist*xdist + .01*ydist*ydist;
	    /* Strength varies inversely with the square of the distance. */
	    float strength = EXPLOSION_STRENGTH / distSquared;
	    float angle = atan2(ydist, xdist);
	    
	    strength = bound(strength, 0, 10*PER_SIDE_POLYROID_VELOCITY);
	    polyroid->dx += strength * cos(angle);
	    polyroid->dy += strength * sin(angle);
	    polyroid->dTheta = 0;      /* Also stop everyone from spinning. */
	    if (!polyroid->dead) {
	      polyroid->numSides -= 1;   /* Reduce # of sides. */
	      if (polyroid->numSides < 3) {
		KillPolyroid(polyroid);
	      }
	    }
	  }
	  ship.concussions--;
	  concussionOn = CONCUSSION_STEPS;  /* We use concussionOn as both
					       a counter and a flag. */
	}
      }
      break;
    case ESCKEY:  /* Quit the current game, going back to attract mode. */
      EndGame();
      break;
    default:
      break;
    }
  }
}

 

void Configure(void)
{
  /* Open and configure a window. */

//   foreground();

  glutInitWindowPosition(XPOS, YPOS);
  glutInitWindowSize(XSIZE, YSIZE);

  glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH);

  Curwin = squarewin = glutCreateWindow("Polyroids");

  glMatrixMode(GL_MODELVIEW);                       /* Set matrix mode. */
//  glEnable(GL_POLYGON_SMOOTH);
   
 /* noise(TIMER0, 1); */

//FIX!
//  glutReshapeFunc(Reshape);
  glutKeyboardFunc(Key);
  glutSpecialFunc(SpecialKey);
  glutKeyboardUpFunc(KeyUp);
  glutSpecialUpFunc(SpecialKeyUp);
  glutDisplayFunc(Display);
  glutIdleFunc(Animate);
  glutVisibilityFunc(Visible);


  /* Setup initial viewing coordinates */
  ortho2(-0.5, XSIZE+0.5, -0.5, YSIZE+0.5 );

  /* Initialize draw data & font manager. */
  DrawInit();

  /* Record starting time. */
  gettimeofday(&lastFrameTime, 0);
  srand(lastFrameTime.tv_sec);

  /* Initialize the game. */
  InitPolyroids((rand() % MAX_POLYROIDS) / 2 + 1);
  InitShip();
  ReadHighScores();
  SetMessage(0, 0.0, "");
  SetMessage(1, 0.0, "");

}

void GetPos(Polyroid *polyroid)
{
  int pos = rand() & 7;

  switch (pos) {
    case 0: {
      polyroid->centerX = 0;
      polyroid->centerY = 0;
    } break;
    case 1: {
      polyroid->centerX = 0;
      polyroid->centerY = YSIZE / 2;
    } break;
    case 2: {
      polyroid->centerX = 0;
      polyroid->centerY = YSIZE;
    } break;
    case 3: {
      polyroid->centerX = XSIZE / 2;
      polyroid->centerY = 0;
    } break;
    case 4: {
      polyroid->centerX = XSIZE / 2;
      polyroid->centerY = YSIZE;
    } break;
    case 5: {
      polyroid->centerX = XSIZE;
      polyroid->centerY = 0;
    } break;
    case 6: {
      polyroid->centerX = XSIZE;
      polyroid->centerY = YSIZE / 2;
    } break;
    case 7: {
      polyroid->centerX = XSIZE;
      polyroid->centerY = YSIZE;
    } break;
  }
}


/* Makes a new, random polyroid. */
Polyroid *MakeNewPolyroid(void)
{
  Polyroid *polyroid = malloc(sizeof(Polyroid));

  polyroid->kind = rand() & 3;

 // printf("new kind %d\n",polyroid->kind);
    
  polyroid->radius = POLYROID_RADIUS;
  polyroid->angle = 0;
  GetPos(polyroid);
  polyroid->dx = rand() & 0xff;
  polyroid->dy = rand() & 0xff;
  polyroid->dTheta = 0;
  polyroid->ddx = 0;
  polyroid->ddy = 0;
  polyroid->ddTheta = ((rand() & 3) + 1) * PI/10;
  polyroid->numSides = (rand() & 3) + 3;
  polyroid->dead = FALSE;
  polyroid->beingHit = FALSE;
/*  polyroid->chaseFactor = (rand() & 1023) + 1;*/
  polyroid->chaseFactor = ((float) (rand() & 0xFFFF)) / 0xFFFF;
  polyroid->powerup = POW_NEVER;
  
  return polyroid;
}


int NextPowerup(int current)
{
  switch (current) {
    case POW_LASER: {
      return POW_CONCUSSION;
    } break;
    case POW_CONCUSSION: {
      return POW_REGEN;
    } break;
    case POW_REGEN: {
      return POW_LASER;
    } break;
    case POW_NEVER: {
      return POW_NEVER;
    } break;
    default: exit(-1); break;
  }
}



void InitPolyroids(int number)
{
  int i;

  /* Free any old polyroids. */
  for (i = 0; i < numPolyroids; i++) {
    free(polyroids[i]);
  }
  
  numPolyroids = numLivePolyroids = 0;
  numNewPolyroidsNeeded = number;
  needNewPowerup = TRUE;
}


void InitShip(void)
{
  int i;

  ship.centerX = XSIZE / 2;
  ship.centerY = YSIZE / 2;
  ship.angle = 0;
  ship.dx = ship.dy = ship.dTheta = 0;
  ship.ddx = ship.ddy = 0;
  ship.score = 0;
  ship.concussions = INITIAL_CONCUSSIONS;
  ship.shields = 1.0;
  ship.shotStrength = BASE_SHOT_STRENGTH;
  ship.shotThrust = BASE_SHOT_THRUST;
  ship.regenRate = BASE_REGEN_RATE;

  thrustOn = turningLeft = turningRight = FALSE;
  for (i = 0; i < NUM_KINDS; i ++) laserOn[i] = FALSE;
  ClearLaserStack();
  currentLaser = -1;

  wave = 1;
  numWavePolyroids = 0;
}


/* Reads high scores from a file. */
void ReadHighScores(void)
{
  FILE *fp;
  int i;
  char *name = "Nobody\n";
  HSNode *current = NULL;
  char filename[100];

  sprintf(filename, "%s/%s", SCORES_DIR, SCORES_FILE);

  fp = fopen(filename, "r");

  /* If file is readable, deallocate old score list. */
  if (fp) {
    current = firstScore;
    while (current) {
      HSNode *temp = current->next;
      free(current);
      current = temp;
    }
  }
  firstScore = NULL;

  for (i = 0; i < NUM_HIGH_SCORES; i++) {
    HSNode *node = malloc(sizeof(HSNode));

    if (fp) {
      char buf[1024];

      fgets(buf, 1024, fp);
      node->name = malloc(strlen(buf)+1);
      strcpy(node->name, buf);
      fgets(buf, 1024, fp);
      node->score = atoi(buf);
    } else {
      node->name = malloc(strlen(name)+1);
      strcpy(node->name, name);
      node->score = 0;
    }

    node->prev = current;
    if (!firstScore) {
      firstScore = node;
    } else {
      current->next = node;
    }
    current = node;
  }
  current->next = NULL;
  lastScore = current;
  if (fp) fclose(fp);
  else WriteHighScores();
}


void WriteHighScores(void)
{
  FILE *fp;
  HSNode *node = NULL;
  char filename[100];

  sprintf(filename, "%s/%s", SCORES_DIR, SCORES_FILE);

  fp = fopen(filename, "w");

  if (fp) {
    for (node = firstScore; node; node = node->next) {
      fprintf(fp, "%s%d\n", node->name, node->score);
    }
    fclose(fp);
  }
}


void EndGame(void)
{
  HSNode *node;
  int i;

  attractMode = TRUE;
  /* Hmmm, should read high scores here. */
  ReadHighScores();

  for (node = firstScore; node; node = node->next) {
    if (ship.score > node->score) {
      HSNode *temp = malloc(sizeof(HSNode));
      uid_t id = getuid();
      struct passwd *pw = getpwuid(id);
      char buf[1024];
      
      if (!pw) {
	fprintf(stderr, "Error reading pw info\n");
	break;
      }

      /* Install new score entry. */
      temp->score = ship.score;
      strcpy(buf, pw->pw_name);
      strcat(buf, "\n");
      temp->name = malloc(strlen(buf)+1);
      strcpy(temp->name, buf);
      if (node == firstScore) {
	temp->prev = NULL;
	firstScore = temp;
      } else {
	temp->prev = node->prev;
	temp->prev->next = temp;
      }
      temp->next = node;
      node->prev = temp;

      /* Remove and free last score entry. */
      lastScore = lastScore->prev;
      free(lastScore->next->name);
      free(lastScore->next);
      lastScore->next = NULL;

      WriteHighScores();    

      break;
    }
  }

  for (i = 0; i < numPolyroids; i++) {
    polyroids[i]->beingHit = FALSE;    
  }

  SetMessage(0, 0.0, "");
  SetMessage(1, 0.0, "");

  showHelp = FALSE;
}




void KillPolyroid(Polyroid *polyroid)
{
  numLivePolyroids--;
  polyroid->numSides = 3;
  polyroid->dead = TRUE;
  ship.score++;

  switch (polyroid->powerup) {
    case POW_NONE: case POW_NEVER: break;
    case POW_LASER: {
      ship.shotStrength += INC_SHOT_STRENGTH;
      ship.shotThrust += INC_SHOT_THRUST;
      SetMessage(1, 2.0, "LASER POWERUP");
      needNewPowerup = TRUE;
    } break;
    case POW_CONCUSSION: {
      ship.concussions++;
      SetMessage(1, 2.0, "CONCUSSION");
      needNewPowerup = TRUE;
    } break;
    case POW_REGEN: {
      ship.regenRate += INC_REGEN_RATE;
      SetMessage(1, 2.0, "INCREASED SHIELD REGENERATION");
      needNewPowerup = TRUE;
    } break;
  }
  polyroid->powerup = POW_NEVER;  

  if (numLivePolyroids == 0 && numWavePolyroids == 0) {
    char string[1024];
    sprintf(string, "END OF LEVEL %d", wave);
    SetMessage(0, 100.0, string);
  }

}

