/*

  util.c

  Utility functions for the polyroids program.


    Copyright (C) 1993 Rob Shillingsburg
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


*/

#include "polyroids.h"


/*** Interface function definitions: *****************************************/


/* Paints the window black. */
void ClearWindow(void)
{
  if (dead) {
    cpack (RGB_WHITE);
    dead = FALSE;
  } else cpack(RGB_BLACK);
  clear();
}


/* Returns x, bounded by the given values. */
float bound(float x, float bottomBound, float topBound)
{
  if (x > topBound) return topBound;
  else if (x < bottomBound) return bottomBound;
  else return x;
}

/* TRUE if pt is `above' seg. */
static int Above(Segment *seg, short *pt)
{
  return (pt[Y] - seg->pt2[Y]) > seg->slope * (pt[X] - seg->pt2[X]);
}

/* TRUE if the segments intersect. */
int SegmentsIntersect(Segment *seg1, Segment *seg2)
{
  return (Above(seg1, seg2->pt1) != Above(seg1, seg2->pt2)) &&
    (Above(seg2, seg1->pt1) != Above(seg2, seg1->pt2));  
}

/* Returns the slope of the line containing the points. */
/* Gives a large number for vertical lines. */
float CalculateSlope(short *pt1, short *pt2)
{
  float rise = pt1[Y] - pt2[Y];
  float run = pt1[X] - pt2[X];

  if (run == 0) return 2000.0;
  else return rise / run;
}


static int laserStack[5] = {-1};
static int top = 0;

void ClearLaserStack(void)
{
  top = 0;
}

/* Puts whichLaser on top of the laser stack.  We assume whichLaser is
   not already in the stack. */
void PushLaser(int whichLaser)
{
  laserOn[whichLaser] = TRUE;
  laserStack[++top] = whichLaser;
  currentLaser = whichLaser;
}

/* Removes whichLaser from the stack, wherever it may be. */
void PopLaser(int whichLaser)
{
  int i, j;

  laserOn[whichLaser] = FALSE;
  
  for (i = top; i > 0; i--) {
    if (laserStack[i] == whichLaser) {
      for (j = i; j < top; j++) laserStack[j] = laserStack[j+1];
      top--;
    }
  }

  currentLaser = laserStack[top];
}
